package be.iminds.ilabt.jfed.testing.tests.highlevel.config;

import be.iminds.ilabt.jfed.experiment.setup.config.ESpec;
import be.iminds.ilabt.jfed.experiment.setup.config.Provision;
import be.iminds.ilabt.jfed.experiment.setup.config.Slice;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUser;
import be.iminds.ilabt.jfed.lowlevel.user.GeniUserProvider;
import be.iminds.ilabt.jfed.testing.base.ApiTestConfig;
import be.iminds.ilabt.jfed.testing.shared.NodeLoginTestStepConfig;
import be.iminds.ilabt.jfed.testing.shared.Proxy;
import com.fasterxml.jackson.annotation.*;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import io.dropwizard.jackson.Jackson;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.security.PrivateKey;
import java.security.PublicKey;

@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public class ESpecTestConfig implements ApiTestConfig {
    private static final Logger LOG = LoggerFactory.getLogger(ESpecTestConfig.class);

    private static final ObjectMapper MAPPER = Jackson.newObjectMapper();

    @Nonnull private final be.iminds.ilabt.jfed.testing.shared.Proxy amConnectionProxy;
    @Nonnull private final String amVersion;
    @Nonnull private final SSHKeys sshKeys;
    @Nonnull private final Slice slice;
    @Nonnull private final ESpec eSpec;
    @Nonnull private final Provision provision;
    @Nonnull private final WaitForReadyInTest waitForReady;
    @Nonnull private final Cleanup cleanup;

    public ESpecTestConfig() {
        this.amConnectionProxy = new be.iminds.ilabt.jfed.testing.shared.Proxy();
        this.amVersion = "auto";
        this.sshKeys = new SSHKeys();

        this.slice = new Slice();
        this.eSpec = new ESpec();
        this.provision = new Provision();
        this.waitForReady = new WaitForReadyInTest();
        this.cleanup = new Cleanup();
    }

    @JsonCreator
    public ESpecTestConfig(@Nullable @JsonProperty("amConnectionProxy") be.iminds.ilabt.jfed.testing.shared.Proxy amConnectionProxy,
                           @Nullable @JsonProperty("amVersion") String amVersion,
                           @Nullable @JsonProperty("sshKeys") SSHKeys sshKeys,
                           @Nullable @JsonProperty("slice") Slice slice,
                           @Nullable @JsonProperty("eSpec") ESpec eSpec,
                           @Nullable @JsonProperty("provision") Provision provision,
                           @Nullable @JsonProperty("waitForReady") WaitForReadyInTest waitForReady,
                           @Nullable @JsonProperty("cleanup") Cleanup cleanup) {
        this.amConnectionProxy = amConnectionProxy == null ? new ESpecTestConfig().getAmConnectionProxy() : amConnectionProxy;
        this.amVersion = amVersion == null ? new ESpecTestConfig().getAmVersion() : amVersion;
        this.sshKeys = sshKeys == null ? new ESpecTestConfig().getSshKeys() : sshKeys;
        this.slice = slice == null ? new Slice() : slice;
        this.eSpec = eSpec == null ? new ESpec() : eSpec;
        this.provision = provision == null ? new Provision() : provision;
        this.waitForReady = waitForReady == null ? new WaitForReadyInTest() : waitForReady;
        this.cleanup = cleanup == null ? new Cleanup() : cleanup;
    }

    @Override
    @JsonIgnore
    public boolean isValid() {
        boolean res = getAmConnectionProxy().isValid();
        res = res && getSshKeys().isValid();
        boolean amVersionValid = getAmVersion().equals("2") || getAmVersion().equals("3") || getAmVersion().equals("auto");
        if (!amVersionValid)
            throw new IllegalArgumentException("Config Error: Unsupported amVersion: \""+getAmVersion()+"\"");
//        res = res && amVersionValid;

        res = res && getSlice().isValid();
        res = res && getESpec().isValid();
        res = res && getProvision().isValid();
        res = res && getWaitForReady().isValid();
        res = res && getCleanup().isValid();
        if (!getProvision().isEnabled() && !getWaitForReady().isEnabled())
            throw new IllegalArgumentException("Config Error: if the Provision step is disabled, waitForReady should be disabled too.");

        return res;
    }

    @JsonProperty
    @Nonnull
    public be.iminds.ilabt.jfed.testing.shared.Proxy getAmConnectionProxy() {
        return amConnectionProxy;
    }

    @JsonProperty
    @Nonnull
    public String getAmVersion() {
        return amVersion;
    }

    @JsonProperty
    @Nonnull
    public SSHKeys getSshKeys() {
        return sshKeys;
    }


    @JsonProperty
    @Nonnull
    public Slice getSlice() {
        return slice;
    }

    @JsonProperty("eSpec")
    @Nonnull
    public ESpec getESpec() {
        return eSpec;
    }

    @JsonProperty
    @Nonnull
    public Provision getProvision() {
        return provision;
    }

    @JsonProperty
    @Nonnull
    public WaitForReadyInTest getWaitForReady() {
        return waitForReady;
    }

    @JsonProperty
    @Nonnull
    public Cleanup getCleanup() {
        return cleanup;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof ESpecTestConfig)) return false;

        ESpecTestConfig that = (ESpecTestConfig) o;

        if (!amConnectionProxy.equals(that.amConnectionProxy)) return false;
        if (!amVersion.equals(that.amVersion)) return false;
        if (!sshKeys.equals(that.sshKeys)) return false;
        if (!slice.equals(that.slice)) return false;
        if (!eSpec.equals(that.eSpec)) return false;
        if (!provision.equals(that.provision)) return false;
        if (!waitForReady.equals(that.waitForReady)) return false;
        return cleanup.equals(that.cleanup);
    }

    @Override
    public int hashCode() {
        int result = amConnectionProxy.hashCode();
        result = 31 * result + amVersion.hashCode();
        result = 31 * result + sshKeys.hashCode();
        result = 31 * result + slice.hashCode();
        result = 31 * result + eSpec.hashCode();
        result = 31 * result + provision.hashCode();
        result = 31 * result + waitForReady.hashCode();
        result = 31 * result + cleanup.hashCode();
        return result;
    }

    @Nonnull
    @Override
    @JsonIgnore
    public String stringValue() {
        try {
            return MAPPER.writeValueAsString(this);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Failed to covert "+getClass().getName()+" to String", e);
        }
    }

    @Override
    public String toString() {
        return stringValue();
    }

    public NodeLoginTestStepConfig generateNodeLoginTestStepConfig(@Nonnull GeniUserProvider geniUserProvider) {
        NodeLoginTestStepConfig.SSHKeySource sshKeySource;
        PublicKey publicKey;
        PrivateKey privateKey;
        switch (getSshKeys().getSource()) {
            case RANDOM: {
                sshKeySource = NodeLoginTestStepConfig.SSHKeySource.RANDOM;
                publicKey = null;
                privateKey = null;
                break;
            }
            case TEST_USER: {
                sshKeySource = NodeLoginTestStepConfig.SSHKeySource.PROVIDED;
                assert geniUserProvider.isUserLoggedIn();
                GeniUser user = geniUserProvider.getLoggedInGeniUser();
                assert user != null;
                publicKey = user.getPublicKey();
                privateKey = user.getPrivateKey();
                break;
            }
            default: throw new IllegalArgumentException("Unsupported ssh keys source: "+getSshKeys().getSource());
        }


        return new NodeLoginTestStepConfig(sshKeySource,
                publicKey,
                privateKey,
                false,
                new Proxy(), //proxy OFF
                2*60 * 1000l,
                null,
                null);
    }
}
