package be.iminds.ilabt.jfed.experimenter_gui.timeline.view.output;

import be.iminds.ilabt.jfed.experimenter_gui.util.ui.TextAreaMessageConsumer;
import be.iminds.ilabt.jfed.rspec_fx.model.controller.ExperimentCommand;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import javafx.beans.value.ChangeListener;
import javafx.scene.control.Tab;
import javafx.scene.control.TextArea;
import javafx.scene.paint.Color;
import org.controlsfx.glyphfont.FontAwesome;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Objects;

/**
 * @author Cedric
 */
public class CommandOutputTab extends Tab {
    private final TextAreaMessageConsumer outputMessageConsumer;


    private final ExperimentCommand command;
    private final TextArea output;
    private final ChangeListener<Boolean> startedListener;
    private final ChangeListener<Boolean> finishedListener;
    private final ChangeListener<String> outputListener;
    private final ChangeListener<String> errListener;

    public CommandOutputTab(ExperimentCommand c) {
        this.command = c;

        String nodeUniqueId = c.getNodeUniqueID();
        if(nodeUniqueId.contains("@"))
            nodeUniqueId = nodeUniqueId.substring(0, nodeUniqueId.indexOf('@'));

        setText(nodeUniqueId + ": " + c.getTag());
        output = new TextArea();
        output.setEditable(false);
        setContent(output);
        setGraphic(GlyphUtils.createSmallGlyph(FontAwesome.Glyph.TERMINAL, Color.rgb(120, 255, 120)));

        this.outputMessageConsumer = new TextAreaMessageConsumer(output);
        this.outputMessageConsumer.start();

        startedListener = (ov, oldValue, newValue) -> {
            if (!command.isSkipped() && command.isStarted()) {
                outputMessageConsumer.addMessage(getTime() + ":\t" + "Your command has started.\n");
            }
        };
        command.getStartedProperty().addListener(startedListener);

        finishedListener = (ov, oldValue, newValue) -> {
            if (!command.isSkipped() && command.isFinished()) {
                outputMessageConsumer.addMessage(getTime() + ":\t" + "Your command has finished.\n");
            }
        };
        command.getFinishedProperty().addListener(finishedListener);

        outputListener = (ov, oldValue, newValue) -> {
            if (!Objects.equals(newValue, "")) {
                if (oldValue != null && oldValue.length() > 0 && newValue.startsWith(oldValue)) {
                    outputMessageConsumer.addMessage(getTime() + ":\t" + newValue.substring(oldValue.length()));
                } else {
                    outputMessageConsumer.addMessage(getTime() + ":\t" + newValue);
                }
            }
        };

        command.getResultProperty().addListener(outputListener);

        errListener = (ov, oldValue, newValue) -> {
            if (!Objects.equals(newValue, "")) {
                if (oldValue != null && oldValue.length() > 0 && newValue.startsWith(oldValue)) {
                    outputMessageConsumer.addMessage(getTime() + ": ERROR\t" + newValue.substring(oldValue.length()));
                } else {
                    outputMessageConsumer.addMessage(getTime() + ": ERROR\t" + newValue);
                }
            }
        };
        command.getErrorProperty().addListener(errListener);
    }

    private String getTime() {
        Date date = new Date();
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
        return sdf.format(date);
    }

    public void unregister() {
        command.getStartedProperty().removeListener(startedListener);
        command.getFinishedProperty().removeListener(finishedListener);
        command.getResultProperty().removeListener(outputListener);
        command.getErrorProperty().removeListener(errListener);
    }


}
